@@ -9,11 +9,11 @@ VIM::Packager
 
 =head1 VERSION
 
-Version 0.03
+Version 0.05
 
 =cut
 
-our $VERSION = '0.03';
+our $VERSION = '0.05';
 
 =head1 SYNOPSIS
 
@@ -21,7 +21,7 @@ our $VERSION = '0.03';
 
 =head1 DESCRIPTION
 
-** UNDER DEVELOPING **
+** TESTING **
 
 =head1 FUNCTIONS
 
